#!/bin/bash

if [ ! -f spark.phar ]; then
    echo "error: No spark.phar has been built yet?"
    exit 1
fi

if [ -z "$VERSION" ]; then
    VERSION="$(git describe --tags)"
    PATH="$PWD/tools:$PATH"
fi

if [ -z "$VERSION" ]; then
    echo "Could not parse version from git. Export VERSION if you are building manually."
    exit 1
fi

if [ -z $DESTINATION ]; then
    DESTINATION="release/$VERSION"
fi
echo "  DESTINATION=$DESTINATION"

echo "* Preparing release direcory"
mkdir -p $DESTINATION

echo "* Copying files to release directory"
cp spark.phar $DESTINATION/spark.phar
cp README.md $DESTINATION/README.md
cp VERSIONS.md $DESTINATION/VERSIONS.md

echo "* Building dist and source archives"
7z a -tzip "$DESTINATION/spark-$VERSION-dist.zip" spark.phar plugins README.md VERSIONS.md >/dev/null
tar cfz "$DESTINATION/spark-$VERSION-dist.tgz" spark.phar plugins README.md VERSIONS.md
7z a -tzip "$DESTINATION/spark-$VERSION-src.zip" bin src runtime plugins composer.json README.md VERSIONS.md >/dev/null
tar cfz "$DESTINATION/spark-$VERSION-src.tgz" bin src runtime plugins composer.json README.md VERSIONS.md

echo "* Creating makeself installer"
test -d release/tmp && rm -rf release/tmp
mkdir release/tmp
cp -R spark.phar plugins README.md VERSIONS.md release/tmp/
pushd release/tmp &>/dev/null
makeself . ../../$DESTINATION/spark-$VERSION-dist.run "Spark $VERSION" ./spark.phar install &>/dev/null
popd &>/dev/null
rm -rf release/tmp

echo -e "  ┌─[$DESTINATION]"
ls -hl --color=auto $DESTINATION | sed "s/^/  │ /"

